package com.ruoyi.web.yh.prodOrder.service.impl;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageHelper;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.k3cloud.business.prodOrder.constants.K3ProdField;
import com.ruoyi.k3cloud.constants.K3FormIds;
import com.ruoyi.k3cloud.service.K3Service;
import com.ruoyi.web.yh.prodOrder.service.ProdOrderConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.web.yh.prodOrder.mapper.ProdOrderMapper;
import com.ruoyi.web.yh.prodOrder.domain.ProdOrder;
import com.ruoyi.web.yh.prodOrder.domain.ProdOrderVO;
import com.ruoyi.web.yh.prodOrder.domain.ProdOrderQuery;
import com.ruoyi.web.yh.prodOrder.service.ProdOrderService;
import org.springframework.util.CollectionUtils;

/**
 * 生产订单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-29
 */
@Service
@Slf4j
public class ProdOrderServiceImpl implements ProdOrderService
{
    @Autowired
    private ProdOrderMapper prodOrderMapper;

    @Autowired
    private K3Service k3Service;

    @Autowired
    private ProdOrderConverter prodOrderConverter;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private RedisLock redisLock;

    /**
     * 查询生产订单
     *
     * @param id 生产订单主键
     * @return 生产订单
     */
    @Override
    public ProdOrderVO selectProdOrderById(Long id)
    {
        return prodOrderMapper.selectProdOrderById(id);
    }

    /**
     * 查询生产订单列表
     *
     * @param prodOrder 生产订单
     * @return 生产订单
     */
    @Override
    public List<ProdOrderVO> selectProdOrderList(ProdOrderQuery prodOrder)
    {
        return prodOrderMapper.selectProdOrderList(prodOrder);
    }

    /**
     * 新增生产订单
     *
     * @param prodOrder 生产订单
     * @return 结果
     */
    @Override
    public int insertProdOrder(ProdOrder prodOrder)
    {
        prodOrder.setCreateTime(DateUtils.getNowDate());
        return prodOrderMapper.insertProdOrder(prodOrder);
    }

    /**
     * 修改生产订单
     *
     * @param prodOrder 生产订单
     * @return 结果
     */
    @Override
    public int updateProdOrder(ProdOrder prodOrder)
    {
        return prodOrderMapper.updateProdOrder(prodOrder);
    }

    /**
     * 批量删除生产订单
     *
     * @param ids 需要删除的生产订单主键
     * @return 结果
     */
    @Override
    public int deleteProdOrderByIds(Long[] ids)
    {
        return prodOrderMapper.deleteProdOrderByIds(ids);
    }

    /**
     * 删除生产订单信息
     *
     * @param id 生产订单主键
     * @return 结果
     */
    @Override
    public int deleteProdOrderById(Long id)
    {
        return prodOrderMapper.deleteProdOrderById(id);
    }

    @Override
    public int sync() {
        // 查询字段
        List<String> fieldKeys = Arrays.asList(
                K3ProdField.FID,
                K3ProdField.F_BILL_NO,
                K3ProdField.F_DOCUMENT_STATUS,
                K3ProdField.F_CREATE_DATE,
                K3ProdField.F_MODIFY_DATE,
                K3ProdField.F_DESCRIPTION,
                K3ProdField.F_DATE,
                K3ProdField.F_IS_REWORK,
                K3ProdField.F_ROW_ID,
                K3ProdField.F_MEMO_ITEM,
                K3ProdField.F_CONVEY_DATE,
                K3ProdField.F_STATUS,
                K3ProdField.F_WORK_SHOP_ID,
                K3ProdField.F_REQ_SRC,
                K3ProdField.F_BASE_UNIT_QTY,
                K3ProdField.F_QTY,
                K3ProdField.F_UNIT_ID
        );
        // 查询ERP数据
        List<JSONArray> erpList = k3Service.selectList(K3FormIds.PROD_ORDER, fieldKeys,0, 2000);

        // 转为PO
        List<ProdOrder> prodOrderList = prodOrderConverter.toPoByErpList(fieldKeys, erpList);

        // 新增或修改
        return this.syncByBillNo(prodOrderList);
    }

    /**
     * 根据单据编号查询单据
     *
     * @param billNo
     */
    @Override
    public ProdOrderVO selectByBillNo(String billNo) {
        if (StringUtils.isBlank(billNo)) {
            return null;
        }
        ProdOrderQuery query = new ProdOrderQuery();
        query.setEqErpBillNo(billNo);
        return this.selectOne(query);
    }

    /**
     * 查询单个
     *
     * @param query
     * @return
     */
    @Override
    public ProdOrderVO selectOne(ProdOrderQuery query) {
        PageHelper.startPage(1, 1 );
        List<ProdOrderVO> list = this.selectProdOrderList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 新增或修改
     */
    private int syncByBillNo(List<ProdOrder> prodOrderList) {
        int result = 0;

        // 异步操作
        for (ProdOrder prodOrder : prodOrderList) {
            scheduledExecutorService.schedule(() -> {
                this.syncByBillNo(prodOrder);
            }, 0, TimeUnit.SECONDS);
        }

        return result;
    }

    private int syncByBillNo(ProdOrder prodOrder) {
        String errorMsg = "成功";
        try {
            if (prodOrder == null || StringUtils.isBlank(prodOrder.getErpBillNo())) {
                throw new ServiceException("参数错误");
            }
            String lockKey = prodOrder.getErpRowId();
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.SYNC_PROD_BILL, lockKey), "当前生产订单正在处理中，请稍后重试");

            try {
                // 若数据库中没有,则新增,否则修改
                ProdOrderVO old = this.selectByErpRowId(prodOrder.getErpRowId());
                if (old == null) {
                    return this.insertProdOrder(prodOrder);
                } else {
                    prodOrder.setId(old.getId());
                    return this.updateProdOrder(prodOrder);
                }
            } finally {
                redisLock.unlock(RedisLockKey.SYNC_PROD_BILL, lockKey);
            }
        } catch (Exception e) {
            errorMsg = e.getMessage();
            return 0;
        } finally {
            // TODO 记录日志
            log.info("同步生产订单:{}",  errorMsg);
        }
    }

    private ProdOrderVO selectByErpRowId(String erpRowId) {
        if (StringUtils.isBlank(erpRowId)) {
            return null;
        }
        ProdOrderQuery query = new ProdOrderQuery();
        query.setEqErpRowId(erpRowId);
        return selectOne(query);
    }

    private ProdOrderVO selectByErpId(String erpId) {
        ProdOrderQuery query = new ProdOrderQuery();
        query.setEqErpId(erpId);
        return this.selectOne(query);
    }
}
